<?php
#####################################################################################################
#
#					Module pour la plateforme de paiement PayZen
#						Version : 2.9a (révision 32759)
#									########################
#					Développé pour Prestashop
#						Version : 1.2.4.0
#						Compatibilité plateforme : V1
#									########################
#					Développé par Lyra Network
#						http://www.lyra-network.com/
#						25/01/2012
#						Contact : support@payzen.eu
#
#####################################################################################################

if(!defined('_PS_ROOT_DIR_'))
{
	include_once(dirname(__FILE__).'/vads_api.php');
}
else
{
	include_once(_PS_ROOT_DIR_ . '/modules/vads/vads_api.php');
}

class VADS extends PaymentModule
{
	private $_html = '';

	public function __construct()
	{
		$this->name = 'vads';
		$this->tab = 'Payment';
		$this->version = '2.9a';
		$this->currencies = false;
//		$this->currencies_mode = 'radio';

		parent::__construct();

		$this->displayName = $this->l('vads-type gateway');
		$this->description = $this->l('Pay with vads-type gateway');
	}

	private function _getAdminParameters() {
		//NB : names are 32 chars max
		return array(
			'VADS_SITE_ID'				=> '12345678',
			'VADS_LIEN_BANQUE'			=> 'https://secure.payzen.eu/vads-payment/',
			'VADS_MODE'					=> 'TEST',
			'VADS_DELAY'				=> '',
			'VADS_MONNAIE'				=> '978',
			'VADS_LANGAGE_DEFAUT'		=> 'fr',
			'VADS_CARTE_DISPO_ALL'		=> '1',
			'VADS_CARTE_DISPO_AMEX'		=> '',
			'VADS_CARTE_DISPO_CB'		=> '',
			'VADS_CARTE_DISPO_MASTER'	=> '',
			'VADS_CARTE_DISPO_VISA'		=> '',
			'VADS_AMOUNT_MIN'			=> '',
			'VADS_AMOUNT_MAX'			=> '',
			'VADS_TYPE_VALIDATION'		=> '',
			'VADS_URL_DEFAUT'			=> 'http://'.htmlspecialchars($_SERVER['HTTP_HOST'], ENT_COMPAT, 'UTF-8').__PS_BASE_URI__.'modules/'.$this->name.'/validation.php',
			'VADS_KEY_TEST'				=> '1111111111111111',
			'VADS_KEY_PROD'				=> '2222222222222222',
			'VADS_RETURN_MODE'			=> 'GET',
			'VADS_REDIRECT_ENABLED'		=> 'False',
			'VADS_REDIRECT_SUCCESS_T'	=> 5,
			'VADS_REDIRECT_SUCCESS_M'	=> 'Redirection vers la boutique dans quelques instants',
			'VADS_REDIRECT_ERROR_T'		=> 5,
			'VADS_REDIRECT_ERROR_M'		=> 'Redirection vers la boutique dans quelques instants'
		);
	}

	public function	install()
	{
		if (!parent::install()
			|| !$this->registerHook('payment')
			|| !$this->registerHook('paymentReturn'))
			return false;

		foreach($this->_getAdminParameters() as $name => $default)
			Configuration::updateValue($name,$default);

		return true;
	}

	public function	uninstall()
	{
		foreach($this->_getAdminParameters() as $name => $default)
			Configuration::deleteByName($name);

		return parent::uninstall();
	}

	/**
	 * Admin form management
	 */
	public function getContent()
	{
		if (Tools::isSubmit('submitVADS'))
			$this->postProcess();

		$this->_adminForm();
		return $this->_html;
	}

	/**
	 * Update admin parameters
	 */
	public function postProcess()
	{
		$vars = $this->_getAdminParameters();
		foreach ($vars as $var => $default) {
			$value = array_key_exists($var, $_REQUEST) ? $_REQUEST[$var] : null;
			Configuration::updateValue($var, $value);
		}
		echo '<div class="conf confirm"><img src="../img/admin/ok.gif" />'.$this->l('Settings updated').'</div>';
	}

	/**
	 * Builds the html code for the admin form
	 */
	public function _adminForm()
	{
		$submit_button = '<div class="clear"><input type="submit" class="button" name="submitVADS" value="'.$this->l('Save').'" /></div>';

		// Form beginning
		$this->_html .= '<form action="'.$_SERVER['REQUEST_URI'].'" method="post">';
		$this->_html .= $submit_button.'<br/>';
		$this->_html .= '
			<fieldset>
				<legend><img src="../modules/'.$this->name.'/logo.gif" alt="logo"/>'.$this->displayName.'</legend>
				'.$this->l('Developped by').' : <b><a href="http://www.lyra-network.com/" target="_blank">Lyra-Network</a></b><br/>
				'.$this->l('Contact email').' : <b>support@payzen.eu</b><br/>
				'.$this->l('Module version').' : <b>2.9a</b><br/>
				'.$this->l('Compatible with payment gateway').' : <b>V1</b><br/>
				'.$this->l('Tested with prestashop version').' : <b>1.2.4.0</b>
			</fieldset>
			<div class="clear">&nbsp;</div>';

		/*
		 * General configuration
		 */
		$this->_html .= '<fieldset><legend>'.$this->l('Payment gateway access').'</legend>';
			$this->_html .= $this->_adminFormTextinput('VADS_SITE_ID', $this->l('Site id'), $this->l('Site id provided by the payment gateway'));
			$this->_html .= $this->_adminFormTextinput('VADS_KEY_TEST', $this->l('Test certificate'), $this->l('Certificate provided by the gateway'));
			$this->_html .= $this->_adminFormTextinput('VADS_KEY_PROD', $this->l('Production certificate'), $this->l('Certificate provided by the gateway'));
			// Mode select
			$options = array('TEST'=>$this->l('Test'), 'PRODUCTION'=>$this->l('Production'));
			$selected = Configuration::get('VADS_MODE');
			$this->_html .= $this->_adminFormSelect($options, $selected, 'VADS_MODE', $this->l('Mode'), $this->l('Test or production mode'));
			$this->_html .= $this->_adminFormTextinput('VADS_LIEN_BANQUE', $this->l('Gateway url'), $this->l('Url the client will be redirected to'), 'size="65"');
		$this->_html .= '</fieldset><div class="clear">&nbsp;</div>';

		/*
		 * Payment settings
		 */
		$this->_html .= '<fieldset><legend>'.$this->l('Payment page').'</legend>';
			// currency
//			$this->_html .= $this->_adminFormTextinput('VADS_MONNAIE', $this->l('Currency'),
//				$this->l('Currency code as defined in standard').' <a href="http://www.iso.org/iso/support/currency_codes_list-1.htm" target="_blank" >ISO 4217</a> ; 978=euro'
//			);
			// langs
			$options = array(
				'de'=>$this->l('German'),
				'en'=>$this->l('English'),
				'es'=>$this->l('Spanish'),
				'fr'=>$this->l('French'),
				'it'=>$this->l('Italian'),
				'ja'=>$this->l('Japanese'),
				'zh'=>$this->l('Chinese'),
				'pt'=>$this->l('Portuguese')
			);
			asort($options);
			$selected = array_key_exists(Configuration::get('VADS_LANGAGE_DEFAUT'), $options) ? Configuration::get('VADS_LANGAGE_DEFAUT') : 'fr';
			$this->_html .= $this->_adminFormSelect($options, $selected, 'VADS_LANGAGE_DEFAUT', $this->l('Language'),
				$this->l('Default language for the payment gateway')
			);

			// delay
			$this->_html .= $this->_adminFormTextinput('VADS_DELAY', $this->l('Delay'), $this->l('Delay before banking (in days)'));

			// validation mode
			$options = array(
				'' => $this->l('Default'),
				'0' => $this->l('Automatic'),
				'1' => $this->l('Manual')
			);
			$selected = array_key_exists(Configuration::get('VADS_TYPE_VALIDATION'), $options) ? Configuration::get('VADS_TYPE_VALIDATION') : '';
			$this->_html .= $this->_adminFormSelect($options, $selected, 'VADS_TYPE_VALIDATION', $this->l('Payment validation'),
				$this->l('If manual is selected, you will have to confirm payments manually in your bank backoffice')
			);

			// cards
			$cards = array('ALL'=>$this->l('All'),'AMEX'=>$this->l('American express'),'CB'=>$this->l('CB'), 'MASTER'=>$this->l('Mastercard'), 'VISA'=>$this->l('Visa'));
			$this->_html .= '<label for="VADS_CARTE_DISPO">'.$this->l('Card type').'</label>
					<div class="margin-form">';
			foreach($cards as $value => $label) {
				$name = 'VADS_CARTE_DISPO_'.$value;
				$this->_html .= '<input type="checkbox" name="'.$name.'" value="1"';
				$this->_html .= Configuration::get($name) ? ' checked="checked"' : '';
				$this->_html .= '/>'.$label.'<br/>';
			}
			$this->_html .= '<p>'.$this->l('Check "all" to display all card types allowed by your contract').'</p></div>';
		$this->_html .= '</fieldset><div class="clear">&nbsp;</div>';

		/*
		 * Deactivate for specific amounts
		 */
		$this->_html .= '<fieldset><legend>'.$this->l('Amount restrictions').'</legend>';
			$this->_html .= $this->_adminFormTextinput('VADS_AMOUNT_MIN', $this->l('Minimum amount'));
			$this->_html .= $this->_adminFormTextinput('VADS_AMOUNT_MAX', $this->l('Maximum amount'));
		$this->_html .= '</fieldset><div class="clear">&nbsp;</div>';

		/*
		 * Return
		 */
		$this->_html .= '<fieldset><legend>'.$this->l('Return to shop').'</legend>';
			// Automatic redirection
			$options = array('False'=>$this->l('Disabled'),'True'=> $this->l('Enabled'));
			$selected = array_key_exists(Configuration::get('VADS_REDIRECT_ENABLED'), $options) ? Configuration::get('VADS_REDIRECT_ENABLED') : 'False';
			$this->_html .= $this->_adminFormSelect($options, $selected, 'VADS_REDIRECT_ENABLED', $this->l('Automatic redirection'),
				$this->l('Redirect the client to the shop at the end of the payment process')
			);
			$this->_html .= $this->_adminFormTextinput('VADS_REDIRECT_SUCCESS_T', $this->l('Success timeout'), $this->l('Time before the client is redirected after a successful payment'), '');
			$this->_html .= $this->_adminFormTextinput('VADS_REDIRECT_SUCCESS_M', $this->l('Success message'), $this->l('Message displayed before redirection after a successful payment'), 'size="65"');
			$this->_html .= $this->_adminFormTextinput('VADS_REDIRECT_ERROR_T', $this->l('Failure timeout'), $this->l('Time before the client is redirected after a failed payment'), '');
			$this->_html .= $this->_adminFormTextinput('VADS_REDIRECT_ERROR_M', $this->l('Failure message'), $this->l('Message displayed before redirection after a failed payment'), 'size="65"');

			// Return mode
			$options = array('GET'=>$this->l('GET (parameters in url)'), 'POST'=>$this->l('POST (parameters in a form)'));
			$selected = array_key_exists(Configuration::get('VADS_RETURN_MODE'), $options) ? Configuration::get('VADS_RETURN_MODE') : 'GET';
			$this->_html .= $this->_adminFormSelect($options, $selected, 'VADS_RETURN_MODE', $this->l('Return mode'),
				$this->l('How the client will transmit the payment result')
			);

			// Urls
			$this->_html .= $this->_adminFormTextinput('VADS_URL_DEFAUT', $this->l('Default url'), $this->l('Default return url'), 'size="65"');
			$this->_html .= '<label>'.$this->l('Check url to copy in your bank backoffice').'</label>
					<div class="margin-form">
						<p>http://'.htmlspecialchars($_SERVER['HTTP_HOST'], ENT_COMPAT, 'UTF-8').__PS_BASE_URI__.'modules/'.$this->name.'/validation.php</p>
					</div>';
		$this->_html .= '</fieldset><div class="clear">&nbsp;</div>';
		$this->_html .= $submit_button;
		$this->_html .= '</form>';
	}

	/**
	 *
	 */
	function _adminFormTextinput($name, $label, $description=null, $extra_attributes='') {
		$value = Configuration::get($name);
		$html  = "\n";
		$html .= '<label for="'.$name.'">'.$label.'</label>';
		$html .= '<div class="margin-form">';
		$html .= '<input type="text" id="'.$name.'" name="'.$name.'" value="'.$value.'" '.$extra_attributes.'/>';
		$html .= '<p>'.$description.'</p>';
		$html .= '</div>';
		return $html;
	}

	function _adminFormSelect($options, $selected, $name, $label, $description) {
		$html = "\n";
		$html .= '<label for="'.$name.'">'.$label.'</label>';
		$html .= '<div class="margin-form">';
		$html .= '<select name="'.$name.'">';
		foreach($options as $value => $label) {
			$html .= '<option value="'.$value.'"';
			$is_selected = is_array($selected) ? in_array($value,$selected)	: ((string)$value==(string)$selected);
			$html .= $is_selected ? ' selected="selected"' : '';
			$html .= '>'.$label.'</option>';
		}
		$html .= '</select><p>'.$description.'</p></div>';
		return $html;
	}

	/**
	 * Payment function, redirects the client to payment page
	 */
	public function hookPayment($params)
	{
		global $smarty;
		$cookie = $params['cookie'];
		$cart = $params['cart'];
		$address = new Address($cart->id_address_invoice);
		$cust = new Customer(intval($cookie->id_customer));
		$pays = new Country($address->id_country);

		$min = Configuration::get('VADS_AMOUNT_MIN');
		$max = Configuration::get('VADS_AMOUNT_MAX');
		if( ($min!='' && $cart->getOrderTotal()<$min) || ($max!='' && $cart->getOrderTotal()>$max) )
			return;

		if(!Configuration::get('VADS_CARTE_DISPO_ALL')){
			$card  = (Configuration::get('VADS_CARTE_DISPO_AMEX') ? 'AMEX' : '');
			$card .= (Configuration::get('VADS_CARTE_DISPO_CB') ? ( $card!='' ? ';' : '' ).'CB' : '');
			$card .= (Configuration::get('VADS_CARTE_DISPO_MASTER') ? ( $card!='' ? ';' : '' ).'MASTERCARD' : '');
			$card .= (Configuration::get('VADS_CARTE_DISPO_VISA') ? ( $card!='' ? ';' : '' ).'VISA' : '');
		}
		else {
			$card = '';
		}

		$arrayLang = array('fr','de','en','it','zh','ja','es');

		$langage = strtolower(Language::getIsoById(intval($cookie->id_lang)));
		if(!in_array($langage,$arrayLang)) {
			$langage = Configuration::get('VADS_LANGAGE_DEFAUT');
		}

		$vads_api = new VADS_API();

		/* Currency detection and conversion */
		//TODO support de toutes les devises grâce au système de conversion de prestashop <!> actuellement impossible, trop de bugs prestashop
		// get cart and module currencies
		$currency_cart = new Currency(intval($cart->id_currency));
//		$currency_module = $this->getCurrency();
		// if module currency is not supported, don't propose this payment mode
		$currency_num = $vads_api->convertAlphaCurrency($currency_cart->iso_code);
		if(!$currency_num) {
//			$currency_module = Currency::getCurrency(Currency::getIdByIsoCode('EUR'));
//			$currency_num = '978';
			return;
		}
// 		// convert if needed (based on paypal module with changed cookie fix)
//		$convert_needed = ($currency_order->id != $currency_module->id);
//		if ($convert_needed) {
//			// only known reliable convert method : change cart currency
//			$currency_save = $cookie->id_currency;
//			$cookie->id_currency = $currency_module->id;
//			$cart->id_currency = $currency_module->id;
//			$cart->update();
//		}
		$amount = $cart->getOrderTotal();
//		if($convert_needed) {
//			// cart currency back to original
//			$cookie->id_currency = $currency_save;
//			$cart->id_currency = $currency_save;
//			$cart->update();
//		}

		$vads_api->set('platform_url',		Configuration::get('VADS_LIEN_BANQUE'));
		$vads_api->set('version',			'V1');
		$vads_api->set('key_test',			Configuration::get('VADS_KEY_TEST'));
		$vads_api->set('key_prod',			Configuration::get('VADS_KEY_PROD'));
		$vads_api->set('ctx_mode',			Configuration::get('VADS_MODE'));
		$vads_api->set('amount', 			round($amount*100));
		$vads_api->set('capture_delay', 	Configuration::get('VADS_DELAY'));
		$vads_api->set('currency', 			$currency_num);
		$vads_api->set('cust_email',		$cust->email);
		$vads_api->set('payment_cards',		$card);
		$vads_api->set('payment_config',	'SINGLE');
		$vads_api->set('site_id',			Configuration::get('VADS_SITE_ID'));
		$vads_api->set('validation_mode',	Configuration::get('VADS_TYPE_VALIDATION'));
		$vads_api->set('cust_id',			$cust->id);
		$vads_api->set('cust_name',			$cust->lastname.' '.$cust->firstname);
		$vads_api->set('cust_address',		$address->address1.' '.$address->address2);
		$vads_api->set('cust_zip',			$address->postcode);
		$vads_api->set('cust_city',			$address->city);
		$vads_api->set('cust_phone',		$address->phone);
		$vads_api->set('cust_country',		$pays->iso_code);
		$vads_api->set('language',			$langage);
		$vads_api->set('order_id',			$cart->id);
		if(isset($cookie->SoColissimo)) {
			$vads_api->set('order_info',	$cookie->SoColissimo);
		}
		$vads_api->set('url_return',		Configuration::get('VADS_URL_DEFAUT'));
		$vads_api->set('return_mode',		Configuration::get('VADS_RETURN_MODE'));
		$vads_api->set('redirect_enabled',	Configuration::get('VADS_REDIRECT_ENABLED'));
		$vads_api->set('redirect_success_timeout',	Configuration::get('VADS_REDIRECT_SUCCESS_T'));
		$vads_api->set('redirect_success_message',	Configuration::get('VADS_REDIRECT_SUCCESS_M'));
		$vads_api->set('redirect_error_timeout',	Configuration::get('VADS_REDIRECT_ERROR_T'));
		$vads_api->set('redirect_error_message',	Configuration::get('VADS_REDIRECT_ERROR_M'));
		$vads_api->set('contrib',			'Prestashop1.2.4.0_2.9a');

		$form = $vads_api->getRequestHtmlForm('id="vads_form" class="hidden"');

		$smarty->assign('vads_form', $form);
		return $this->display(__FILE__, 'ordervads.tpl');
	}

	/**
	 * Manage payement gateway response
	 * @param array $params
	 */
	public function hookPaymentReturn($params)
	{
		global $smarty;
		if (!$this->active)
			return ;

		switch (@$_GET['error'])
		{
			case null:
			case '' :
				$error_msg = false;
				break;
			case 'cancel_on_accepted' :
				$error_msg = $this->l('Invalid payment code received but order has already been registered');
				break;
			case 'check_url_warning':
				$error_msg = $this->l('Order registered, but automatic confirmation has not been received. Have you properly configured the check url in the bank backoffice ?');
				break;
			default:
				$error_msg = $this->l('An error occured, your order has not been registered.');
		}

		if($error_msg===false)
		{
			$smarty->assign(array(
				'total_to_pay' => Tools::displayPrice($params['total_to_pay'], $params['currencyObj'], false, false),
				'status' => 'ok',
				'id_order' => $params['objOrder']->id
			));
		}
		else
		{
			$smarty->assign(array(
				'status' => 'failed',
				'error_msg' => $error_msg)
			);
		}
		return $this->display(__FILE__, 'payment_return.tpl');
	}
}
?>